Attribute VB_Name = "modGeom"
'******************************************************************'
'*                                                                *'
'*                      TurboCAD for Windows                      *'
'*                   Copyright (c) 1993 - 2001                    *'
'*             International Microcomputer Software, Inc.         *'
'*                            (IMSI)                              *'
'*                      All rights reserved.                      *'
'*                                                                *'
'******************************************************************'

Option Explicit

Sub BasicGeometryAnalysis()

'This sample shows how to analyze geometry of various objects in
'a TurboCAD drawing. Of course it is a very simple example  you
'can write more sophisticated routines to analyze complex objects,
'to pass this data to other applications or store data in a database, etc.

'The code in this macro sets reference to the active TurboCAD drawing,
'then creates and displays two objects: a circle and an ellipse
'(AddCircleCenterAndPoint and AddEllipse methods of the Grapghics object),
'and waits for user mouse click to determine the type of the object
'user clicked on (GetMouseClick method of the View object) and
'calculate the object's area. If a click occurs on blank area,
'the procedure exist displaying a warning message. If user clicks
'on a circular object, the procedure displays a message box
'containing the object's type and area.
  
'Declare variables to hold TurboCAD objects
  Dim ActDr As Drawing
  Dim Grs As Graphics
  Dim Gr As Graphic
  Dim Vi As View
  Dim GrType As String
  
'Assign PI constant
  Const Pi# = 3.14159
    
'Get reference to active drawing, its graphics collection,
'and active view
    Set ActDr = ActiveDrawing
    Set Grs = ActDr.Graphics
    Set Vi = ActDr.Views(0)
    
'Draw circle
    Set Gr = Grs.AddCircleCenterAndPoint(5, 5, 0, 7, 5, 0)
'and use Draw method of Graphic object to display the object
    Gr.Draw
    
'Draw ellipse
    Set Gr = Grs.AddEllipse(7, 7, 0, 10, 9, 0)
'and use Draw method of Graphic object to display the object
    Gr.Draw

'Declare PickResult collection to hold graphics located
'at the point of mouse click
Dim PRes As PickResult

'Declare variables to hold mouse pointer coordinates
'and graphic items at the point of mouse click
Dim xClick#, yClick#
Dim PicCount As Long

'We ignore object-specific errors in this sample
  On Error Resume Next
  
'Get mouse click coordinates
    Vi.GetMouseClick xClick, yClick
    Set PRes = Vi.PickPoint(xClick, yClick)
    
'Get number of items inside PicResult collection
    PicCount = PRes.Count
    
'Check if any object is selected and exit if not
    If PicCount = 0 Then
        MsgBox "No object selected!", vbExclamation
        Exit Sub
    End If

'Get first graphic item in the collection
    Set Gr = PRes(0).Graphic
    
'Check if the graphic type is Circle or Arc
    If Gr.Type = "CIRCLE" Or GrType = "ARC" Then

'Declare AcrData array to hold the graphic (Circle, Arc) geometry paramerters
      Dim ArcData(10) As Double
          Gr.GetArcData ArcData

'Get the graphics center point coordinates
      Dim xc#, yc#
          xc = ArcData(0)
          yc = ArcData(1)
'Get ratio
      Dim Ratio#
          Ratio = ArcData(6)
          
'Determine if it's a closed graphic
      Dim Closed%
          Closed = ArcData(9)
          
'Determine circular object type
      Dim CircleType$
          If Ratio = 1 And Closed = 1 Then CircleType = "Circle"
          If Ratio <> 1 And Closed = 1 Then CircleType = "Ellipse"
          If Ratio = 1 And Closed = 0 Then CircleType = "ArcCircle"
          If Ratio <> 1 And Closed = 0 Then CircleType = "ArcEllipse"
      
'Determine main axis' coordinates
      Dim xMajor#, yMajor#
          xMajor = ArcData(3)
          yMajor = ArcData(4)
      
'Calculate major and minor axes
      Dim a#, b#
          a = Sqr((xMajor - xc) * (xMajor - xc) + (yMajor - yc) * (yMajor - yc))
          b = Ratio * a
      
'Calculate object's area and display a message with results
      Dim Area#
          Area = 0
          If CircleType = "Circle" Then
              Area = Pi * a * a
              MsgBox "This is a circle. It's area is " & Round(Area, 2) & _
              " sq. inches", vbInformation
          End If
          If CircleType = "Ellipse" Then
              Area = Pi * a * b
              MsgBox "This is an ellipse . It's area is " & Round(Area, 2) & _
              " sq. inches", vbInformation
          End If
    Else
      MsgBox "This is not a circular object or object isn't closed!", vbExclamation
    End If

'Destroy objects
  Set ActDr = Nothing
  Set Grs = Nothing
  Set Vi = Nothing
  Set Gr = Nothing
  Set PRes = Nothing

End Sub

