Attribute VB_Name = "modViewport"
'******************************************************************'
'*                                                                *'
'*                      TurboCAD for Windows                      *'
'*                   Copyright (c) 1993 - 2001                    *'
'*             International Microcomputer Software, Inc.         *'
'*                            (IMSI)                              *'
'*                      All rights reserved.                      *'
'*                                                                *'
'******************************************************************'

Option Explicit

Const strNamedView = "FloorPlan"

Public Sub SetModelSpace()
    On Error Resume Next
    With ActiveDrawing
        Dim Space%
        Space = .Properties("TileMode")
        If Space <> 1 Then
            .Properties("TileMode") = 1
        End If
    End With
End Sub

Public Sub SetPaperSpace()
    On Error Resume Next
    With ActiveDrawing
        Dim Space%
        Space = .Properties("TileMode")
        If Space <> 0 Then
            .Properties("TileMode") = 0
        End If
    End With
End Sub

Public Function FindNamedView(ByVal viewName As String) As NamedView
    On Error GoTo FindError
    Dim count As Long, i As Long
    Dim nvws As NamedViews
    Dim nvw As NamedView
    
    Set nvws = ActiveDrawing.NamedViews
    count = nvws.count
    For i = 0 To count - 1
        Set nvw = nvws(i)
        If nvw.name = viewName Then
            Set FindNamedView = nvw
            Exit Function
        End If
    Next
FindError:
    Set FindNamedView = Nothing
End Function

Public Sub CreateVpNamedView()
    On Error Resume Next
    Dim nvw As NamedView
    Set nvw = FindNamedView(strNamedView)
    
    'Only create if it doesn't exist
    If nvw Is Nothing Then
        With ActiveDrawing
            'Set to model space to set up named view
            SetModelSpace
        
            'Add the named view
            Set nvw = .NamedViews.Add(strNamedView, 0, 0, 100, 100)
        End With
    End If
End Sub

Public Sub DeleteVpNamedView()
    On Error Resume Next
    Dim nvw As NamedView
    Set nvw = FindNamedView(strNamedView)
    If Not nvw Is Nothing Then
        nvw.Delete
    End If
End Sub

Sub AddViewport()
    On Error Resume Next
    
    'Create named view if it doesn't exist already
    CreateVpNamedView
    
    'Obtain world coordinates of viewport center and size
    'Drawing must be in paper space mode
    SetPaperSpace
    Dim vx#, vy#, vz#
    Dim wx1#, wy1#, wx2#, wy2#, wz#
    Dim gr1 As Graphic, gr2 As Graphic
    With ActiveDrawing.ActiveView
        MsgBox "Click to define the first corner of the viewport"
        .GetMouseClick vx, vy
        .ViewToWorld vx, vy, vz, wx1, wy1, wz
        
        'Place corner marker as a helper
        Set gr1 = ActiveDrawing.Graphics.AddCross(wx1, wy1, 0)
        .Refresh
        
        MsgBox "Click to define the second corner of the viewport"
        .GetMouseClick vx, vy
        .ViewToWorld vx, vy, vz, wx2, wy2, wz
        
        'Place corner marker as a helper
        Set gr2 = ActiveDrawing.Graphics.AddCross(wx2, wy2, 0)
        
        'Add the viewport graphic with correct scaling
        AddViewportByName strNamedView, wx1, wy1, wx2, wy2
        
        'Remove temporary corner markers and show the results
        gr1.Delete
        Set gr1 = Nothing
        gr2.Delete
        Set gr2 = Nothing
        .Refresh
    End With
End Sub

Sub AddViewportByName(ByVal viewName As String, _
    ByVal x1 As Double, ByVal y1 As Double, ByVal x2 As Double, ByVal y2 As Double)
    On Error Resume Next
    Dim BBox As BoundingBox
    
    'Get the named view that we're going to use
    Dim nvw As NamedView
    Set nvw = FindNamedView(viewName)
    If nvw Is Nothing Then
        MsgBox "Named view " & viewName & " not found--can't create viewport"
        Exit Sub
    End If
    
    'Get bounding box size info from named view
    Set BBox = nvw.BoundingBox
    Dim nvwWidth#, nvwHeight#
    nvwWidth = BBox.Max.X - BBox.Min.X
    nvwHeight = BBox.Max.Y - BBox.Min.Y
    Set BBox = Nothing
    
    'Drawing must be in paper space mode
    'Viewport graphic goes on paper sapce
    SetPaperSpace
    
    'Add viewport graphic and set it up
    Dim vpg As Graphic
    Set vpg = ActiveDrawing.Graphics.Add(imsiViewport)
    With vpg
    
        'Set the NamedView property
        .Properties("NamedView") = viewName
        
        'Get bounding box size and location info from viewport graphic
        Set BBox = .CalcBoundingBox
        Dim vpgWidth#, vpgHeight#, vpgCenterX#, vpgCenterY#
        vpgWidth = BBox.Max.X - BBox.Min.X
        vpgHeight = BBox.Max.Y - BBox.Min.Y
        vpgCenterX = (BBox.Max.X + BBox.Min.X) / 2
        vpgCenterY = (BBox.Max.Y + BBox.Min.Y) / 2
        Set BBox = Nothing

        'Calculate desired center point
        Dim centerX#, centerY#
        centerX = (x1 + x2) / 2
        centerY = (y1 + y2) / 2
        
        'Calculate scale factors
        'Choose smallest size for scale
        Dim sizeX#, sizeY#, sc#, scX#, scY#
        If x1 = x2 Then
            sizeX = 1
        Else
            sizeX = Abs(x2 - x1)
        End If
        If y1 = y2 Then
            sizeY = 1
        Else
            sizeY = Abs(y2 - y1)
        End If
        
        If sizeX < sizeY Then
            scY = sizeX / vpgHeight
            scX = sizeX / vpgWidth
        Else
            scX = sizeY / vpgWidth
            scY = sizeY / vpgHeight
        End If
        
        'Move viewport graphic to specified location
        .MoveRelative centerX - vpgCenterX, centerY - vpgCenterY, 0
        
        'Now centered at centerX, centerY
        'Scale (resize) viewport graphic around center
        Dim mtx As Matrix
        Set mtx = .Scale(scX, scY, 1, centerX, centerY, 0)
    End With
    Exit Sub
End Sub

